from __future__ import unicode_literals
import uuid

from pyramid.path import DottedNameResolver


def set_version(request, response):
    """Set version and revision to response

    """
    settings = request.registry.settings
    resolver = DottedNameResolver()

    # get version config
    version_header = settings.get(
        'api.version_header',
        'X-Version',
    )
    version_header_value = settings.get('api.version_header_value')
    if callable(version_header_value):
        version_header_value = version_header_value()
    elif version_header_value:
        version_header_value = resolver.resolve(version_header_value)

    # get revision config
    revision_header = settings.get(
        'api.revision_header',
        'X-Revision',
    )
    revision_header_value = settings.get('api.revision_header_value')
    if callable(revision_header_value):
        revision_header_value = revision_header_value()
    elif revision_header_value:
        revision_header_value = resolver.resolve(revision_header_value)

    if version_header and version_header_value:
        response.headers[str(version_header)] = str(version_header_value)
    if revision_header and revision_header_value:
        response.headers[str(revision_header)] = str(revision_header_value)


def set_req_guid(request, response):
    settings = request.registry.settings
    # get revision config
    req_guid_header = settings.get(
        'api.req_guid_header',
        'X-Req-Guid',
    )
    if not req_guid_header:
        return

    # TODO: what about security issue? what about attacker feeds us a
    # mal-format req_guid or what? a hash signed guid instead?
    req_guid = request.headers.get(req_guid_header)
    if req_guid is None:
        req_guid = uuid.uuid4().hex
    if not req_guid:
        return
    response.headers[str(req_guid_header)] = str(req_guid)


def api_headers_tween_factory(handler, registry):
    """This tween provides necessary API headers

    """

    def api_headers_tween(request):
        response = handler(request)
        set_version(request, response)
        set_req_guid(request, response)
        return response

    return api_headers_tween



from __future__ import unicode_literals

import base64
import binascii


def get_remote_user(request):
    """Parse basic HTTP_AUTHORIZATION and return user name

    """
    if 'HTTP_AUTHORIZATION' not in request.environ:
        return
    authorization = request.environ['HTTP_AUTHORIZATION']
    try:
        authmeth, auth = authorization.split(' ', 1)
    except ValueError:  # not enough values to unpack
        return
    if authmeth.lower() != 'basic':
        return
    try:
        auth = base64.b64decode(auth.strip().encode('utf8')).decode('utf8')
    except (binascii.Error, TypeError):  # can't decode
        return
    try:
        login, password = auth.split(':', 1)
    except ValueError:  # not enough values to unpack
        return
    return login, password


def basic_auth_tween_factory(handler, registry):
    """Do basic authentication, parse HTTP_AUTHORIZATION and set remote_user
    variable to request

    """
    def basic_auth_tween(request):
        remote_user = get_remote_user(request)
        if remote_user is not None:
            request.environ['REMOTE_USER'] = remote_user[0]
        return handler(request)
    return basic_auth_tween



from __future__ import unicode_literals

import six


def allow_origin_tween_factory(handler, registry):
    """Allow cross origin XHR requests

    """
    def allow_origin_tween(request):
        settings = request.registry.settings
        request_origin = request.headers.get('origin')

        def is_origin_allowed(origin):
            allowed_origins = (
                request.registry.settings.get('api.allowed_origins', [])
            )
            if isinstance(allowed_origins, six.string_types):
                allowed_origins = allowed_origins.splitlines()
            if not origin:
                return False
            for allowed_origin in allowed_origins:
                if origin.lower().startswith(allowed_origin):
                    return True
            return False

        def allow_origin_callback(request, response):
            """Set access-control-allow-origin et. al headers

            """
            allowed_methods = settings.get(
                'api.allowed_methods',
                str('GET, POST, PUT, DELETE, PATCH, OPTIONS'),
            )
            if callable(allowed_methods):
                allowed_methods = allowed_methods(request)

            allowed_headers = settings.get(
                'api.allowed_headers',
                str('Content-Type, Authorization, Range'),
            )
            if callable(allowed_headers):
                allowed_headers = allowed_headers(request)

            allowed_credentials = settings.get(
                'api.allowed_credentials',
                str('true'),
            )
            if callable(allowed_credentials):
                allowed_credentials = allowed_credentials(request)

            response.headers['Access-Control-Allow-Origin'] = request_origin
            if allowed_credentials:
                response.headers[str('Access-Control-Allow-Credentials')] = \
                    str(allowed_credentials)
            if allowed_methods:
                response.headers[str('Access-Control-Allow-Methods')] = str(
                    allowed_methods,
                )
            if allowed_headers:
                response.headers[str('Access-Control-Allow-Headers')] = str(
                    allowed_headers,
                )

        if not is_origin_allowed(request_origin):
            return handler(request)

        request.add_response_callback(allow_origin_callback)
        return handler(request)
    return allow_origin_tween



from ez_setup import use_setuptools
use_setuptools()

from setuptools import setup, find_packages

version = '0.0.0'
try:
    import pyramid_handy
    version = pyramid_handy.__version__
except ImportError:
    pass

tests_require = [
    'webtest',
    'pytest',
    'pytest-cov',
    'pytest-xdist',
    'pytest-capturelog',
]

setup(
    name='pyramid-handy',
    description='Some handy stuff for Pyramid web framework',
    classifiers=[
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Intended Audience :: Developers",
        "Programming Language :: Python",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
    ],
    keywords='pyramid',
    author='Victor Lin',
    author_email='hello@victorlin.me',
    url='https://github.com/victorlin/pyramid-handy',
    license='MIT',
    version=version,
    packages=find_packages(),
    install_requires=[
        'pyramid',
        'six',
    ],
    extras_require=dict(
        tests=tests_require,
    ),
    tests_require=tests_require,
)
